<?php
// $Id: component.inc,v 1.4 2007/04/16 21:41:14 ff1 Exp $

/**
 * @file
 * Enables users to create a league in the fantasy formula 1 game.
 */


/**
 * Implementation of hook_node_name().
 */
function ff1_component_node_name($node) {
  return t('component');
}

/**
 * Implementation of hook_access().
 */
function ff1_component_access($op, $node) {
  if ($op == 'create' || $op == 'update' || $op == 'delete') {
    return user_access('administer nodes');
  }
}

/**
 * Implementation of hook_form().
 */
function ff1_component_form(&$node) {
  if (isset($node->title)) {
    if ($node->comp_type == 'driver') {
      $cars = ff1_component_arrayload('car');
      $output .= form_select(t('Car'), 'car', $node->car, $cars, NULL, 0, FALSE, TRUE);
    } elseif ($node->comp_type == 'car') {
      $engines = ff1_component_arrayload('engine');
      $output .= form_select(t('Engine'), 'engine', $node->engine, $engines, NULL, 0, FALSE, TRUE);
    }
  }

  $output = '';

  if (function_exists('taxonomy_node_form')) {
    $output .= implode('', taxonomy_node_form('ff1_component', $node));
  }

  $output .= form_select(t('Type'), 'comp_type', $node->comp_type, drupal_map_assoc(array('car','driver','engine')), NULL, 0, FALSE, TRUE);
  $output .= form_textarea(t('Profile'), 'body', $node->body, 60, 5, '', NULL, FALSE);
  // extra relationship: driver-car or car-engine
  if (isset($node->title)) {
    if ($node->comp_type == 'driver') {
      $cars = ff1_component_arrayload('car');
      $output .= form_select(t('Car'), 'car', $node->car, $cars, NULL, 0, FALSE, TRUE);
    } elseif ($node->comp_type == 'car') {
      $engines = ff1_component_arrayload('engine');
      $output .= form_select(t('Engine'), 'engine', $node->engine, $engines, NULL, 0, FALSE, TRUE);
    }
  }
  $output .= form_textfield(t("Nationality"), "nationality", $node->nationality, 50, 50, NULL, NULL, FALSE);
  $output .= form_textfield(t("Web Site"), "website", $node->website, 50, 255, t("Example: http://www.mysite.com"), NULL, FALSE);
  $output .= form_textfield(t("Price"), "price", $node->price, 10, 3, t("Example: 5 = &pound;5million, 20 = &pound;20million"), NULL, TRUE);
  $output .= filter_form('format', $node->format);

  return $output;
}

/**
 * Implementation of hook_validate().
 */
function ff1_component_validate(&$node) {
  if (isset($node->website) && $node->website!='' && !valid_url($node->website)) {
    form_set_error('website', t("The address specified is not a valid website address."));
  }
  if (isset($node->price) && $node->price=='') {
    form_set_error('price', t("Please specify a price."));
  }
  if (isset($node->price) && $node->price!='' && !is_numeric($node->price)) {
    form_set_error('price', t("Please specify a numeric price."));
  }
}

/**
 * Implementation of hook_load().
 */
function ff1_component_load($node) {
  $component = db_fetch_object(db_query("SELECT * FROM {ff1_components} WHERE nid = '%d'", $node->nid));
  if ($component->comp_type == 'driver') {
    $component->car = db_result(db_query("SELECT child_nid FROM {ff1_relationships} WHERE nid = '%d'", $node->nid));
  } elseif ($component->comp_type == 'car') {
    $component->engine = db_result(db_query("SELECT child_nid FROM {ff1_relationships} WHERE nid = '%d'", $node->nid));
  }
  return $component;
}

/**
 * Implementation of hook_insert().
 */
function ff1_component_insert($node) {
  db_query("INSERT INTO {ff1_components} (nid, comp_name, comp_type, nationality, website, price, aka) VALUES (%d, '%s', '%s', '%s', '%s', %d, '%s')", $node->nid, $node->title, $node->comp_type, $node->nationality, $node->website, $node->price, $node->title);

  // add in extra relationship: driver-car or car-engine
  if ($node->comp_type == 'driver' && isset($node->car)) {
    db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver_car')", $node->nid, $node->car);
  } elseif ($node->comp_type == 'car' && isset($node->engine)) {
    db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'car_engine')", $node->nid, $node->engine);
  }
}

/**
 * Implementation of hook_update().
 */
function ff1_component_update($node) {
  db_query("UPDATE {ff1_components} SET comp_name = '%s', comp_type = '%s', nationality = '%s', website = '%s', price = %d, aka = '%s' WHERE nid = %d", $node->title, $node->comp_type, $node->nationality, $node->website, $node->price, $node->title, $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE nid = %d", $node->nid);
  // add in extra relationship: driver-car or car-engine
  if ($node->comp_type == 'driver' && isset($node->car)) {
    db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver_car')", $node->nid, $node->car);
  } elseif ($node->comp_type == 'car' && isset($node->engine)) {
    db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'car_engine')", $node->nid, $node->engine);
  }
}

/**
 * Implementation of hook_delete().
 */
function ff1_component_delete($node) {
  db_query("DELETE FROM {ff1_components} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE child_nid = %d", $node->nid);
}

/**
 * Implementation of hook_view().
 */
function ff1_component_view(&$node, $main = 0, $page = 0) {
  $node = ff1_component_content($node, $page);
  if ($page) {
    // Breadcrumb navigation
    $breadcrumb[] = l(t('Home'), NULL);
    $breadcrumb[] = l(t('components'), 'components');
    $breadcrumb[] = l($node->comp_type . 's', 'components/' . $node->comp_type . 's');
    drupal_set_breadcrumb($breadcrumb);
  }
  return theme('node', $node, $main, $page);
}

/**
 * Implementation of hook_content().
 */
function ff1_component_content(&$node, $page = 0) {
  if ($page) {
    // Full page
    $output .= theme('form_element', t('Type'), $node->comp_type);
    if ($node->body) $output .= theme('form_element', t('Profile'), check_output($node->body, $node->format));
    if ($node->car) {
      $output .= theme('form_element', t('Car'), l(ff1_component_get_name($node->car),'node/' . $node->car));
    } elseif ($node->engine) {
      $output .= theme('form_element', t('Engine'), l(ff1_component_get_name($node->engine),'node/' . $node->engine));
    }
    if ($node->nationality) $output .= theme('form_element', t('Nationality'), $node->nationality);
    if ($node->website) $output .= theme('form_element', t('Website'), l($node->website, $node->website));
    $output .= theme('form_element', t('Price'), t('&pound;') . $node->price . t('million'));
    // Display components points
    $result = db_query(db_rewrite_sql("
SELECT r.race_name, s.session, s.position, c.comp_name, s.score, s.driver_nid, d.comp_name driver_name
FROM {ff1_component_scores} s
INNER JOIN {ff1_races} r ON s.race_nid = r.nid
INNER JOIN {ff1_components} c ON s.comp_nid = c.nid
INNER JOIN {ff1_components} d ON s.driver_nid = d.nid
AND s.comp_nid = %d
ORDER BY r.race_number, s.session, s.position
"), $node->nid);
    if (db_num_rows($result)) {
      // Put results into a table
      // Set up table header
      $header = array(
        t('Race'),
        t('Session'),
        t('Position'),
        (($node->comp_type == 'driver') ? '' : t('Driver')),
        t('Points')
      );
      while ($points = db_fetch_object($result)) {
        if ($points->session == 'both_finish') {
           $session = t('bonus');
           $points->position = '';
           $points->driver_name = t('both cars finished');
        } elseif ($points->session == 'fastest_lap') {
           $session = t('fastest lap');
           $points->position = '';
        } else {
           $session = $points->session;
        }
        // Set up table rows
        $rows[] = array(
          $points->race_name,
          $session,
          $points->position,
          (($node->comp_type == 'driver') ? '' : l($points->driver_name,'node/' . $points->driver_nid)),
          $points->score
        );
      }
      $total_score = db_result(db_query("SELECT SUM(score) FROM {ff1_component_scores} WHERE comp_nid = %d", $node->nid));
      $rows[] = array(array('data' => t('<b>Total score</b>'), 'colspan' => '4', 'align' => 'right'), $total_score);
      $output .= theme('box', t('Points so far'), theme('table', $header, $rows));
    }

    $node->body = '<div class="content">'. $output . '</div></div>';
    $node->teaser = $node->body;
  }
  else {
    // Short page
    $node->teaser = check_output($node->teaser,$node->format);
  }
  $node->readmore = true;
  return $node;
}

/**
 * Custom function to return an array of components of a specific type.
 * The key is the component nid and the value is the component name.
 * This is mainly used for displaying lists on forms.
 */
function ff1_component_arrayload($comp_type, $price = FALSE) {
  $components = array();
  $result = db_query("SELECT nid, comp_name, price FROM {ff1_components} WHERE comp_type = '%s' ORDER BY price DESC", $comp_type);
  while ($comp = db_fetch_object($result)) {
    // Turn results into an associative array
    $components[$comp->nid] = $comp->comp_name;
    if ($price) $components[$comp->nid] .= ' (' . $comp->price . ')';
  }
  return $components;
}

/**
 * Custom function to return the component name from its nid.
 */
function ff1_component_get_name($nid) {
  return db_result(db_query("SELECT comp_name FROM {ff1_components} WHERE nid = '%d'", $nid));
}

/**
 * Custom function to list the components.
 */
function ff1_component_list($type = 'all') {
  if ($type == 'drivers') {
    $type = 'driver';
  } elseif ($type == 'cars') {
    $type = 'car';
  } elseif ($type == 'engines') {
    $type = 'engine';
  } else {
    $type = 'all';
  }
  // Breadcrumb navigation
  $breadcrumb[] = l(t('Home'), NULL);
  if ($type == 'all') {
    drupal_set_title('components');
  } else {
    $breadcrumb[] = l(t('components'), 'components');
  }
  drupal_set_breadcrumb($breadcrumb);

  if ($type == 'all') {
    $output = theme('box', 'drivers', _ff1_component_get_table('driver'));
    $output .= theme('box', 'cars', _ff1_component_get_table('car'));
    $output .= theme('box', 'engines', _ff1_component_get_table('engine'));
  } else {
    $output = _ff1_component_get_table($type);
  }
  print theme('page', $output);
}

/**
 * Custom function to list the components.
 */
function _ff1_component_get_table($type) {
  // Get the components from the db
  $result = db_query(db_rewrite_sql("SELECT c.*, rc.comp_name child FROM {ff1_components} c LEFT JOIN {ff1_relationships} r ON c.nid = r.nid LEFT JOIN {ff1_components} rc ON r.child_nid = rc.nid WHERE c.comp_type = '%s' ORDER BY c.price DESC"), $type);

  if (!db_num_rows($result)) {
    return t('No %comps have been created.', array('%comp' => $type));
  }

  // Get the column header for the child column depending on the type of component
  if ($type == 'driver') {
    $column_header = t('Car');
  } elseif ($type == 'car') {
    $column_header = t('Engine');
  } else {
    $column_header = '';
  }

  // Create a table containing each of the components.
  while ($comp = db_fetch_object($result)) {
    // Set up table rows
    $rows[] = array(
      $comp->nid,
      l($comp->comp_name, 'node/' . $comp->nid),
      $comp->nationality,
      $comp->child,
      $comp->price,
    );
  };

  // Set up table header
  $header = array(
    t('ID'),
    t('Name'),
    t('Nationality'),
    $column_header,
    t('Price')
  );

  // Generate a table using: theme('table', $header, $rows)
  return theme('table', $header, $rows);
}

/**
 * Custom function to list the components.
 */
function ff1_component_scores_list($type = 'all') {
  if ($type == 'drivers') {
    $type = 'driver';
  } elseif ($type == 'cars') {
    $type = 'car';
  } elseif ($type == 'engines') {
    $type = 'engine';
  } else {
    $type = 'all';
  }
  // Breadcrumb navigation
  $breadcrumb[] = l(t('Home'), NULL);
  if ($type == 'all') {
    drupal_set_title('scores');
  } else {
    $breadcrumb[] = l(t('scores'), 'scores');
  }
  drupal_set_breadcrumb($breadcrumb);

  // tell the user if the results are still provisional
  if (variable_get('ff1_result_status', t('Official')) == t('Provisional')) drupal_set_message(t('### PROVISIONAL ###'));

  if ($type == 'all') {
    $output = theme('box', 'drivers', _ff1_component_scores_get_table('driver'));
    $output .= theme('box', 'cars', _ff1_component_scores_get_table('car'));
    $output .= theme('box', 'engines', _ff1_component_scores_get_table('engine'));
  } else {
    $output = _ff1_component_scores_get_table($type);
  }
  print theme('page', $output);
}

/**
 * Custom function to list the components.
 */
function _ff1_component_scores_get_table($type) {
  // Get the last race
  $race = variable_get('ff1_last_race', 0);
  // Get the components from the db
  $result = db_query(db_rewrite_sql("SELECT c.comp_name, c.nid, s.score
FROM {ff1_components} c
INNER JOIN {ff1_component_sum_scores} s ON c.nid = s.comp_nid
WHERE s.race_nid = %d
AND c.comp_type = '%s'
GROUP BY c.nid
ORDER BY s.score DESC"), $race, $type);

  if (!db_num_rows($result)) {
    return t('No %comps scores are available at this moment.', array('%comp' => $type));
  }

  // Create a table containing each of the components.
  while ($comp = db_fetch_object($result)) {
    // Set up table rows
    $rows[] = array(
      $comp->nid,
      l($comp->comp_name, 'node/' . $comp->nid),
      $comp->score
    );
  };

  // Set up table header
  $header = array(
    t('ID'),
    t('Name'),
    t('Score')
  );

  // Generate a table using: theme('table', $header, $rows)
  return theme('table', $header, $rows);
}

?>