<?php
// $Id: league.inc,v 1.4 2007/04/16 21:41:14 ff1 Exp $

/**
 * @file
 * Enables users to create a league in the fantasy formula 1 game.
 */


/**
 * Implementation of hook_node_name().
 */
function ff1_league_node_name($node) {
  return t('league');
}

/**
 * Implementation of hook_access().
 */
function ff1_league_access($op, $node) {
  global $user;

  if ($op == 'create') {
    return user_access('create leagues');
  }

  if ($op == 'update' || $op == 'delete') {
    if (user_access('edit own leagues') && ($user->uid == $node->uid)) {
      return TRUE;
    }
  }
}

/**
 * Implementation of hook_form().
 */
function ff1_league_form(&$node) {
  $output = '';

  if (function_exists('taxonomy_node_form')) {
    $output .= implode('', taxonomy_node_form('ff1_league', $node));
  }

  $password_text = t('Enter a password twice if you want to make this a private league, or leave it blank to make this a public league.');
  $password_text .= '<br />';
  $password_text .= t('Users will need to enter this password in order to join your private league. A user can enter a public league without supplying a password.');

  $output .= form_textarea(t('Profile'), 'body', $node->body, 60, 5, '', NULL, TRUE);
  $output .= form_group('Private League',
    form_password(t('Password'), 'pass1', '', 12, 24, '', NULL, FALSE) .
    form_password(t('Confirm Password'), 'pass2', '', 12, 24, '', NULL, FALSE),
    $password_text);

  $output .= filter_form('format', $node->format);

  return $output;
}

/**
 * Implementation of hook_load().
 */
function ff1_league_load($node) {
  // Add the league details to the league node
  $league = db_fetch_object(db_query("SELECT * FROM {ff1_leagues} WHERE nid = '%d'", $node->nid));
  return $league;
}

/**
 * Implementation of hook_insert().
 */
function ff1_league_insert($node) {
  db_query("INSERT INTO {ff1_leagues} (nid, league_name, password, private) VALUES (%d, '%s', '%s', %d)", $node->nid, $node->title, $node->password, $node->private);
}

/**
 * Implementation of hook_update().
 */
function ff1_league_update($node) {
  db_query("UPDATE {ff1_leagues} SET league_name = '%s', password = '%s', private = %d WHERE nid = %d", $node->title, $node->password, $node->private, $node->nid);
}

/**
 * Implementation of hook_delete().
 */
function ff1_league_delete($node) {
  db_query("DELETE FROM {ff1_leagues} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE child_nid = %d", $node->nid);
}

function ff1_league_validate(&$node) {
  // If required, check that proposed passwords match.
  // If so, encrypt the new password and add it to $node.
  if ($node->pass1) {
    $node->pass1 = trim($node->pass1);
    $node->pass2 = trim($node->pass2);
    if ($node->pass1 == $node->pass2) {
      $node->password = md5($node->pass1);
      $node->private = 1;
    }
    else {
      form_set_error('pass2', t('The specified passwords do not match.'));
    }
  }
  unset($node->pass1, $node->pass2);

  //Check that the league name does not already exist
  if (db_num_rows(db_query("SELECT nid FROM {node} WHERE type = '%s' AND status = 1 AND title = '%s' AND nid <> %d", $node->type, $node->title, $node->nid))) {
    form_set_error('title', t('This league name is already being used. Please choose a different league name.'));
  }

  return $node;
}

/**
 * Implementation of hook_view().
 */
function ff1_league_view(&$node, $main = 0, $page = 0) {
  $node = ff1_league_content($node, $page);
  if ($page) {
    // Breadcrumb navigation
    $breadcrumb[] = l(t('Home'), NULL);
    $breadcrumb[] = l(t('leagues'), 'league_of_leagues');
    drupal_set_breadcrumb($breadcrumb);
  }
  return theme('node', $node, $main, $page);
}

/**
 * Implementation of hook_content().
 */
function ff1_league_content(&$node, $page = 0) {
  if ($page) {
    // Full page
    $output .= theme('form_element', t('League Name'), $node->league_name);
    $output .= theme('form_element', t('League Owner'), l($node->name,'user/' . $node->uid));
    if ($node->body) $output .= theme('form_element', t('League Profile'), check_output($node->body, $node->format));
    $output .= theme('form_element', t('League Privacy'), ($node->password ? t('This is a Private league') : t('This is a Public league')));
    $output .= theme('form_element', t('League Table'), l(t('View Table'), 'league/table/' . $node->nid));
    $node->body = '<div class="content">'. $output . '</div></div>';
    $node->teaser = $node->body;
  }
  else {
    // Short page
    $node->teaser = check_output($node->teaser,$node->format);
  }
  $node->readmore = true;
  return $node;
}

/**
 * Custom function to display a page where you can your own leagues.
 */
function ff1_league_my_leagues($arg = 0) {
  $current_user = FALSE;
  if ($arg) {
    $user = db_fetch_object(db_query("SELECT uid, name FROM {users} WHERE uid = %d OR name = '%s'", $arg, $arg));
    if ($user) {
      drupal_set_title(t('%user\'s Leagues', array('%user' => $user->name)));
    }
  }

  if (!$user) {
    global $user;
    $current_user = TRUE;
    drupal_set_title(t('My Leagues'));
  }

  if (!$user->uid) {
    print theme('page', t('You need to log in to see your leagues.'));
    return;
  }

  // Set up table header
  $header = array(
    t('league ID'),
    t('league Name'),
    t('league Privacy'),
    t('league Table')
  );

  // use $user->uid to view all leagues owned by user
  $result = db_query(db_rewrite_sql("SELECT * FROM {ff1_leagues} l INNER JOIN {node} n ON n.nid = l.nid WHERE n.uid = '%d'"), $user->uid);

  if (!db_num_rows($result)) {
    $leagues_owned = ($current_user ? t('You have no leagues.') . ' <a href=node/add/ff1_league>Create league</a>' : t('This user has no leagues.'));
  } else {
    while ($node = db_fetch_object($result)) {
      // Set up table rows
      $rows_owned[] = array(
        $node->nid,
        l($node->league_name, 'node/' . $node->nid),
        ($node->password ? t('Private') : t('Public')),
        l(t('View Table'), 'league/table/' . $node->nid)
      );
    }

    // Generate a table using: theme('table', $header, $rows)
    $leagues_owned = theme('table', $header, $rows_owned);
  }

  // use $user->uid to view all leagues entered by user
  $result = db_query(db_rewrite_sql("SELECT DISTINCT l.* FROM {ff1_leagues} l INNER JOIN {ff1_relationships} r ON r.nid = l.nid INNER JOIN {node} n ON n.nid = r.child_nid WHERE n.uid = '%d'"), $user->uid);

  if (!db_num_rows($result)) {
    $leagues_entered = ($current_user ? t('You have not entered any leagues.') : t('%user has not entered any leagues.', array('%user' => $user->name)));
  } else {
    while ($node = db_fetch_object($result)) {
      // Set up table rows
      $rows_entered[] = array(
        $node->nid,
        l($node->league_name, 'node/' . $node->nid),
        ($node->password ? t('Private') : t('Public')),
        l(t('View Table'), 'league/table/' . $node->nid)
      );
    }

    // Generate a table using: theme('table', $header, $rows)
    $leagues_entered = theme('table', $header, $rows_entered);
  }

  $output = theme('box', ($current_user ? t('Leagues I own') : t('Leagues owned by %user', array('%user' => $user->name))), $leagues_owned);
  $output .= theme('box', ($current_user ? t('Leagues I\'ve entered') : t('Leagues entered by %user', array('%user' => $user->name))), $leagues_entered);

  print theme('page', $output);
}

/**
 * Custom function to return an array of leagues.
 * The key is the league nid and the value is the league name.
 * This is mainly used for displaying lists on forms.
 */
function ff1_league_arrayload() {
  $leagues = array();
  $result = db_query("SELECT nid, league_name, private FROM {ff1_leagues} ORDER BY league_name");
  while ($league = db_fetch_object($result)) {
    // Turn results into an associative array
    $leagues[$league->nid] = $league->league_name;
    if ($league->private == 1) $leagues[$league->nid] .= ' (PRIVATE)';
  }
  return $leagues;
}

/**
 * Custom function to display a paged league table.
 *
 * @param $nid
 *   node ID of the league
 *
 * @param $op
 *   -'points_so_far':
 *      Display the league table containing all points scored upto and including the
 *      selected race.
 *   -'points_this_race':
 *      Display the league table containing only points scored during the
 *      selected race.
 *
 * @param $which_race
 *   node ID of the race
 */
function ff1_league_league($nid = 0, $op = 'points_so_far', $which_race = NULL) {
  if ($which_race) {
    if (is_numeric($which_race) && db_num_rows(db_query("SELECT * FROM {ff1_results_formal} WHERE race_nid = %d", $which_race))) {
      $race = $which_race;
    } else {
      // Get the last race
      $race = variable_get('ff1_last_race', 0);
    }
  } else {
    // Get the last race
    $race = variable_get('ff1_last_race', 0);
  }
  // load the node information for the selected race.
  $node_race = node_load(array('nid' => $race));
  switch($op) {
    case 'points_this_race':
      $table_type = 'points_this_race';
      $race_label = t('For the %sentence_name Grand Prix', array('%sentence_name' => $node_race->sentence_name));
      break;
    default:
      $table_type = 'points_so_far';
      $race_label = t('After the %sentence_name Grand Prix', array('%sentence_name' => $node_race->sentence_name));
      break;
  }

  // Get the Tie Breaker value from the db
  $tiebreaker = variable_get('ff1_tiebreaker', 0);

  // Create the sql statement to display the league
  // %league_join and %league_clause are only required if a league nid is specified.
  $sql = "
SELECT t.team_name, t.nid, lt.rank, lt.d1_score driver1, lt.d2_score driver2, lt.car_score car, lt.engine_score engine, lt.team_score
" . (($tiebreaker == 0) ? "" : ", ABS(t.tbvalue - " . $tiebreaker . ") as tberror") . "
FROM {ff1_league_table} lt
INNER JOIN {ff1_teams} t ON lt.team_nid = t.nid
%league_join
WHERE lt.race_nid = %race
AND lt.table_type = '%table_type'
%league_clause
ORDER BY lt.rank ASC, t.team_name ASC";

  // Set the limit of teams to be returned per page
  $limit = 50;

  // Make sure you don't trust the URL to be safe! Always check for exploits. 
  if (!is_numeric($nid)) {
    // We will just show a standard "access denied" page in this case. 
    drupal_access_denied(); 
    return; 
  } elseif ($nid == 0) {
    drupal_set_title(t('League table for \'Main Public League\''));
    //$output .= t('Here is the league table for \'Main Public League\'.');
    // Get the main public league table
    // No league has been specified, so do not use %league_join and %league_clause
    $sql = t($sql, array('%league_join' => '', '%race' => $race, '%table_type' => $table_type, '%league_clause' => ''));
    // For the pager query, we need to find out how many records will be returned.
    $sql_count = "SELECT COUNT(*) FROM {ff1_teams}";
    // Perform the query
    $result = pager_query(db_rewrite_sql($sql), $limit, 0, db_rewrite_sql($sql_count));
  } else {
    $node = node_load(array('nid' => $nid));
    if ($node) {
      drupal_set_title(t('League table for \'%league\'', array('%league' => $node->league_name)));
      // Get the league table
      // $node->nid has been specified so use it to view all teams in this league
      // include %league_join and %league_clause.
      $sql = t($sql, array('%league_join' => "INNER JOIN {ff1_relationships} rl ON t.nid = rl.child_nid", '%race' => $race, '%table_type' => $table_type, '%league_clause' => "AND rl.nid = %d"));
      // For the pager query, we need to find out how many records will be returned.
      $sql_count = "SELECT COUNT(*) FROM {ff1_teams} t INNER JOIN {ff1_relationships} rl ON t.nid = rl.child_nid WHERE rl.nid = %d";
      // Perform the query
      $result = pager_query(db_rewrite_sql($sql), $limit, 0, db_rewrite_sql($sql_count), $node->nid, $node->nid);
    }
  }

  if (!db_num_rows($result)) {
    print theme('page', t('There are no teams in this league.'));
    return;
  }
  $count = $_GET['from'];
  while ($team = db_fetch_object($result)) {
    $count++;
    // Set up table rows
    $rows[] = array(
      ($node ? $count : $team->rank),
      l($team->team_name, 'node/' . $team->nid),
      $team->driver1,
      $team->driver2,
      $team->car,
      $team->engine,
      '<b>' . $team->team_score . '</b>',
      $team->tberror
    );
    // Remove the tie breaker error column if the tiebreaker value is not yet set
    // eg. only show the tie breaker column after the last race of the season
    if ($tiebreaker == 0) {
      $trash = array_pop($rows[count($rows) - 1]);
    }
  }

  // Set up table header
  $header = array(
    t('Position'),
    t('Team Name'),
    t('Driver1'),
    t('Driver2'),
    t('Car'),
    t('Engine'),
    t('Points'),
    t('Tie Break Error')
  );
  // Remove the tie breaker error column if the tiebreaker value is not yet set
  // eg. only show the tie breaker column after the last race of the season
  if ($tiebreaker == 0) {
    $trash = array_pop($header);
  }

  // Generate the pager links at the bottom of the page
  if ($pager = theme('pager', NULL, $limit)) {
    // Add the pager to the bottom of the table
    $rows[] = array(array('data' => $pager, 'colspan' => '7'));
    // Add the pager above the table
    $output .= $pager;
  }

  // Generate a table using: theme('table', $header, $rows)
  $output .= theme('table', $header, $rows);

  // Put this into a box with the race name in the header
  $output = theme('box', $race_label, $output);

  // tell the user if the results are still provisional
  if (variable_get('ff1_result_status', t('Official')) == t('Provisional')) drupal_set_message(t('### PROVISIONAL ###'));

  print theme('page', $output);
}


/**
 * Custom function to display the league of leagues.
 */
function ff1_league_of_leagues() {
  // Get the last race
  $race = variable_get('ff1_last_race', 0);
  // Create the sql statement to display the league
  $sql = "
SELECT l.league_name, l.nid, (@rank := @rank + 1) AS rank, SUM(lt.team_score) AS league_score, COUNT(*) AS teams, SUM(lt.team_score)/COUNT(*) AS average, l.private
FROM {ff1_leagues} l
LEFT JOIN {ff1_relationships} rl ON l.nid = rl.nid
LEFT JOIN {ff1_league_table} lt ON rl.child_nid = lt.team_nid
WHERE lt.race_nid = " . $race . "
AND lt.table_type = 'points_so_far'
GROUP BY l.nid
ORDER BY average DESC, teams DESC, league_name";

/*SELECT l.league_name, l.nid, SUM(ct.score) league_score, COUNT(*)/4 teams, SUM(ct.score)/(COUNT(*)/4) average FROM {ff1_leagues} l
LEFT JOIN {ff1_relationships} rl ON l.nid = rl.nid
LEFT JOIN {ff1_relationships} rt ON rl.child_nid = rt.nid
LEFT JOIN {ff1_component_sum_scores} ct ON rt.child_nid = ct.comp_nid
WHERE ct.race_nid = " . $race . "
GROUP BY l.nid
ORDER BY average DESC";*/
  db_query("SET @rank = 0;");

  drupal_set_title(t('League of leagues'));
  // Get the league table
  // $node->nid has been specified so use it to view all teams in this league
  // include %league_join and %league_clause.
  $result = db_query(db_rewrite_sql($sql), $node->nid);

  if (!db_num_rows($result)) {
    print theme('page', t('There are no leagues.'));
    return;
  }

  while ($league = db_fetch_object($result)) {
    //$count += 1;
    // Set up table rows
    $rows[] = array(
      //$count,
      $league->rank,
      l($league->league_name, 'node/' . $league->nid),
      ($league->private == 1 ? theme('image', 'files/icons/16-security-key.png', 'Private league', 'Private league') : ''),
      '<a href="node/' . $league->nid . '" title="View league information">' . theme('image', 'files/icons/16-message-info.png', 'View league information', 'View league information') . '</a>',
      '<a href="league/table/' . $league->nid . '" title="View league table">' . theme('image', 'files/icons/16-columns.png', 'View league table', 'View league table') . '</a>',
      $league->teams,
      $league->average
    );
  }

  // Set up table header
  $header = array(
    t('Position'),
    t('League Name'),
    '',
    '',
    '',
    t('Teams'),
    t('Average')
  );

  // Generate a table using: theme('table', $header, $rows)
  $output = theme('table', $header, $rows);

  print theme('page', $output);
}

/**
 * Custom function to allow users to select the exact configuration of league table that they wish to view.
 */
function ff1_league_select_view() {

  if (isset($_POST['submit'])) {
    $edit = $_POST['edit'];
    $league = $edit['league'];
    $table_type = $edit['table_type'];
    $race = $edit['race'];
    // send the user to the required league table page.
    drupal_goto("league/table/$league/$table_type/$race");
  }

  drupal_set_title(t('League table selection'));

  // League
  $leagues = array();
  $leagues[0] = t('Main Public League');
  $result = db_query("SELECT nid, league_name, private FROM {ff1_leagues} ORDER BY league_name");
  while ($league = db_fetch_object($result)) {
    // Turn results into an associative array
    $leagues[$league->nid] = $league->league_name;
  }
  $form_output = form_select(t('League'), 'league', 0, $leagues, t('Select which league table you wish to view.'), 0, FALSE, TRUE);
  // Race
  $races = array();
  $result = db_query("SELECT DISTINCT r.nid, r.race_name FROM {ff1_races} r INNER JOIN {ff1_results_formal} f ON r.nid = f.race_nid ORDER BY r.race_number ASC");
  while ($race = db_fetch_object($result)) {
    // Turn results into an associative array
    $races[$race->nid] = $race->race_name;
  }
  $form_output .= form_select(t('Race'), 'race', variable_get('ff1_last_race', 0), $races, t('Select which race you wish to view the league table for.'), 0, FALSE, TRUE);
  // Table type
  $table_types = array('points_so_far' => t('Points gained up to and including the selected race'), 'points_this_race' => t('Points gained only during the selected race weekend'));
  $form_output .= form_radios(t('Table Type'), 'table_type', 'points_so_far', $table_types, NULL, TRUE, NULL);
  // Submit
  $form_output .= form_submit(t('submit'), 'submit');
  $output .= form($form_output);
  print theme('page', $output);
}

/**
 * Custom function to allow users to display any league table on their own site.
 * It generates a javascript file that can be accessed from any web page.
 *
 * @param $nid
 *   node ID of the league
 *
 * @param $op
 *   -'points_so_far':
 *      Display the league table containing all points scored upto and including the
 *      selected race.
 *   -'points_this_race':
 *      Display the league table containing only points scored during the
 *      selected race.
 *
 * @param $which_race
 *   node ID of the race
 */

function ff1_league_feed($nid = 0, $op = 'points_so_far', $which_race = NULL) {
  if ($which_race) {
    if (is_numeric($which_race) && db_num_rows(db_query("SELECT * FROM {ff1_results_formal} WHERE race_nid = %d", $which_race))) {
      $race = $which_race;
    } else {
      // Get the last race
      $race = variable_get('ff1_last_race', 0);
    }
  } else {
    // Get the last race
    $race = variable_get('ff1_last_race', 0);
  }
  // load the node information for the selected race.
  $node_race = node_load(array('nid' => $race));
  switch($op) {
    case 'points_this_race':
      $table_type = 'points_this_race';
      $race_label = t('For the %sentence_name Grand Prix', array('%sentence_name' => $node_race->sentence_name));
      break;
    default:
      $table_type = 'points_so_far';
      $race_label = t('After the %sentence_name Grand Prix', array('%sentence_name' => $node_race->sentence_name));
      break;
  }

  // Get the Tie Breaker value from the db
  $tiebreaker = variable_get('ff1_tiebreaker', 0);

  // Create the sql statement to display the league
  // %league_join and %league_clause are only required if a league nid is specified.
  $sql = "
SELECT t.team_name, t.nid, lt.rank, lt.d1_score driver1, lt.d2_score driver2, lt.car_score car, lt.engine_score engine, lt.team_score
" . (($tiebreaker == 0) ? "" : ", ABS(t.tbvalue - " . $tiebreaker . ") as tberror") . "
FROM {ff1_league_table} lt
INNER JOIN {ff1_teams} t ON lt.team_nid = t.nid
%league_join
WHERE lt.race_nid = %race
AND lt.table_type = '%table_type'
%league_clause
ORDER BY lt.rank ASC, t.team_name ASC";

  // Set the limit of teams to be returned per page
  $limit = 500;

  // Make sure you don't trust the URL to be safe! Always check for exploits. 
  if (!is_numeric($nid)) {
    // We will just show a standard "access denied" page in this case. 
    drupal_access_denied(); 
    return; 
  } elseif ($nid == 0) {
    drupal_set_title(t('League table for \'Main Public League\''));
    //$output .= t('Here is the league table for \'Main Public League\'.');
    // Get the main public league table
    // No league has been specified, so do not use %league_join and %league_clause
    $sql = t($sql, array('%league_join' => '', '%race' => $race, '%table_type' => $table_type, '%league_clause' => ''));
    // For the pager query, we need to find out how many records will be returned.
    $sql_count = "SELECT COUNT(*) FROM {ff1_teams}";
    // Perform the query
    $result = pager_query(db_rewrite_sql($sql), $limit, 0, db_rewrite_sql($sql_count));
  } else {
    $node = node_load(array('nid' => $nid));
    if ($node) {
      drupal_set_title(t('League table for \'%league\'', array('%league' => $node->league_name)));
      // Get the league table
      // $node->nid has been specified so use it to view all teams in this league
      // include %league_join and %league_clause.
      $sql = t($sql, array('%league_join' => "INNER JOIN {ff1_relationships} rl ON t.nid = rl.child_nid", '%race' => $race, '%table_type' => $table_type, '%league_clause' => "AND rl.nid = %d"));
      // For the pager query, we need to find out how many records will be returned.
      $sql_count = "SELECT COUNT(*) FROM {ff1_teams} t INNER JOIN {ff1_relationships} rl ON t.nid = rl.child_nid WHERE rl.nid = %d";
      // Perform the query
      $result = pager_query(db_rewrite_sql($sql), $limit, 0, db_rewrite_sql($sql_count), $node->nid, $node->nid);
    }
  }

  if (!db_num_rows($result)) {
    print theme('page', t('There are no teams in this league.'));
    return;
  }
  $count = $_GET['from'];
  while ($team = db_fetch_object($result)) {
    $count++;
    // Set up table rows
    $rows[] = array(
      ($node ? $count : $team->rank),
      l($team->team_name, $base_url . '/node/' . $team->nid),
      $team->driver1,
      $team->driver2,
      $team->car,
      $team->engine,
      '<b>' . $team->team_score . '</b>',
      $team->tberror
    );
    // Remove the tie breaker error column if the tiebreaker value is not yet set
    // eg. only show the tie breaker column after the last race of the season
    if ($tiebreaker == 0) {
      $trash = array_pop($rows[count($rows) - 1]);
    }
  }

  // Set up table header
  $header = array(
    t('Position'),
    t('Team Name'),
    t('Driver1'),
    t('Driver2'),
    t('Car'),
    t('Engine'),
    t('Points'),
    t('Tie Break Error')
  );
  // Remove the tie breaker error column if the tiebreaker value is not yet set
  // eg. only show the tie breaker column after the last race of the season
  if ($tiebreaker == 0) {
    $trash = array_pop($header);
  }

  // Generate the pager links at the bottom of the page
  if ($pager = theme('pager', NULL, $limit)) {
    // Add the pager to the bottom of the table
    $rows[] = array(array('data' => $pager, 'colspan' => '7'));
    // Add the pager above the table
    $output .= $pager;
  }

  // Generate a table using: theme('table', $header, $rows)
  $output .= theme('table', $header, $rows);

  // tell the user if the results are still provisional
  if (variable_get('ff1_result_status', t('Official')) == t('Provisional')) {
    $race_label .= ' ### PROVISIONAL ###';
  }

  // Put this into a box with the race name in the header
  $output = theme('box', $race_label, $output);

  print "(function(){
    document.write('" . $output . "');
  })()";
}

?>