<?php
// $Id: race.inc,v 1.4 2007/04/16 21:41:14 ff1 Exp $

/**
 * @file
 * Enables users to create a league in the fantasy formula 1 game.
 */


/**
 * Implementation of hook_node_name().
 */
function ff1_race_node_name($node) {
  return t('race');
}

/**
 * Implementation of hook_access().
 */
function ff1_race_access($op, $node) {
  global $user;

  if ($op == 'create') {
    return user_access('administer nodes');
  }

  if ($op == 'update' || $op == 'delete') {
    if (user_access('administer nodes') && ($user->uid == $node->uid)) {
      return TRUE;
    }
  }
}

/**
 * Implementation of hook_form().
 */
function ff1_race_form(&$node) {
  $output = '';

  if (function_exists('taxonomy_node_form')) {
    $output .= implode('', taxonomy_node_form('ff1_race', $node));
  }

  $output .= form_select(t('Race Number'), 'race_number', $node->race_number, drupal_map_assoc(array(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19)), NULL, 0, FALSE, TRUE);
  $output .= form_textarea(t('Profile'), 'body', $node->body, 60, 5, '', NULL, FALSE);
  /**
   * superceded by use of event module
  $output .= form_textfield(t("Date"), "race_date", format_date($node->race_date, 'short'), 50, 50, NULL, NULL, TRUE);
   */
  $output .= form_textfield(t("Track Name"), "track_name", $node->track_name, 50, 50, NULL, NULL, TRUE);
  $output .= form_textfield(t("Sentence Name"), "sentence_name", $node->sentence_name, 50, 50, t("Example: ... won the <em>Japanese</em> grand prix ..."), NULL, TRUE);
  $output .= form_textfield(t("Web Site"), "website", $node->website, 50, 255, t("Example: http://www.mysite.com"), NULL, FALSE);
  $output .= filter_form('format', $node->format);

  return $output;
}

/**
 * Implementation of hook_validate().
 */
function ff1_race_validate(&$node) {
  if (isset($node->website) && $node->website!='' && !valid_url($node->website)) {
    form_set_error('website', t("The address specified is not a valid website address."));
  }
  if (isset($node->track_name) && $node->track_name=='') {
    form_set_error('track_name', t("Please specify a Track Name."));
  }
  if (isset($node->sentence_name) && $node->sentence_name=='') {
    form_set_error('sentence_name', t("Please specify a Sentence Name."));
  }
  /**
   * superceded by use of event module
  if (isset($node->race_date) && $node->race_date=='') {
    form_set_error('race_date', t("Please specify a Date."));
  }
  if (isset($node->race_date) && $node->race_date!='' && !is_numeric($node->race_date)) {
    if (($timestamp = strtotime($node->race_date)) === -1) {
      form_set_error('race_date', t("Please specify a correctly formatted Date."));
    } else {
      $node->race_date = $timestamp;
    }
  }
   */
}

/**
 * Implementation of hook_load().
 */
function ff1_race_load($node) {
  $race = db_fetch_object(db_query("SELECT * FROM {ff1_races} WHERE nid = '%d'", $node->nid));
  return $race;
}

/**
 * Implementation of hook_insert().
 */
function ff1_race_insert($node) {
  db_query("INSERT INTO {ff1_races} (nid, race_number, race_name, race_date, track_name, sentence_name, website) VALUES (%d, %d, '%s', '%s', '%s', '%s', '%s')", $node->nid, $node->race_number, $node->title, $node->race_date, $node->track_name, $node->sentence_name, $node->website);
}

/**
 * Implementation of hook_update().
 */
function ff1_race_update($node) {
  db_query("UPDATE {ff1_races} SET race_number = %d, race_name = '%s', race_date = '%s', track_name = '%s', sentence_name = '%s', website = '%s' WHERE nid = %d", $node->race_number, $node->title, $node->race_date, $node->track_name, $node->sentence_name, $node->website, $node->nid);
}

/**
 * Implementation of hook_delete().
 */
function ff1_race_delete($node) {
  db_query("DELETE FROM {ff1_races} WHERE nid = %d", $node->nid);
}

/**
 * Implementation of hook_view().
 */
function ff1_race_view(&$node, $main = 0, $page = 0) {
  $node = ff1_race_content($node, $page);
  if ($page) {
    // Breadcrumb navigation
    $breadcrumb[] = l(t('Home'), NULL);
    $breadcrumb[] = l(t('races'), 'races');
    drupal_set_breadcrumb($breadcrumb);
  }
  return theme('node', $node, $main, $page);
}

/**
 * Implementation of hook_content().
 */
function ff1_race_content(&$node, $page = 0) {
  if ($page) {
    // Full page
    $output .= theme('form_element', t('Track Name'), $node->track_name . ', ' . $node->title);
    $output .= theme('form_element', t('Race Number'), $node->race_number);
    //$output .= theme('form_element', t('Date'), format_date($node->race_date, 'custom', 'j F Y'));
    if ($node->body) $output .= theme('form_element', t('Profile'), check_output($node->body, $node->format));
    if ($node->website) $output .= theme('form_element', t('Website'), l($node->website, $node->website));
    $node->body = '<div class="content">'. $output . '</div></div>';
    $node->teaser = $node->body;
  }
  else {
    // Short page
    $node->teaser = check_output($node->teaser,$node->format);
  }
  $node->readmore = true;
  return $node;
}

/**
 * Custom function to return an array of races of a specific type.
 * The key is the race nid and the value is the race name.
 * This is mainly used for displaying lists on forms.
 */
function ff1_race_arrayload() {
  $races = array();
  $result = db_query("SELECT nid, race_name FROM {ff1_races} ORDER BY race_number ASC");
  while ($race = db_fetch_object($result)) {
    // Turn results into an associative array
    $races[$race->nid] = $race->race_name;
  }
  return $races;
}

/**
 * Custom function to return the race name from its nid.
 */
function ff1_race_get_name($nid) {
  return db_result(db_query("SELECT race_name FROM {ff1_races} WHERE nid = '%d'", $nid));
}

/**
 * Custom function to list the races.
 */
function ff1_race_list() {
  // Breadcrumb navigation
  $breadcrumb[] = l(t('Home'), NULL);
  drupal_set_breadcrumb($breadcrumb);

  // Get the races from the db
  $result = db_query("SELECT DISTINCT r.*, f.race_nid results, e.event_start
FROM {ff1_races} r
LEFT JOIN {ff1_results_formal} f ON r.nid = f.race_nid
LEFT JOIN {event} e ON r.nid = e.nid
ORDER BY e.event_start");

  if (!db_num_rows($result)) {
    $output =  t('No race information is available.');
  } else {
    // Create a table containing each of the races.
    while ($race = db_fetch_object($result)) {
      // Set up table rows
      $rows[] = array(
        $race->race_number,
        format_date($race->event_start, 'custom', 'j F H:i'),
        l($race->race_name, 'node/' . $race->nid),
        $race->track_name,
        '<a href="' . $race->website . '"><img src="files/flags/' . $race->flag . '.gif" alt="' . $race->race_name . '" /></a>',
        ($race->results ? l(t('Results'), 'results/' . $race->nid) : '')
      );
    };

    // Set up table header
    $header = array(
      t('No.'),
      t('Date'),
      t('Race'),
      t('Track'),
      t('Website'),
      t('Results')
    );

    // Generate a table using: theme('table', $header, $rows)
    $output = theme('box', t('2007 race calendar'), theme('table', $header, $rows));
  }
  print theme('page', $output);
}

/**
 * Custom function to display the race results.
 */
function ff1_race_results($race_nid = 0) {
  if ($race_nid == 0) {
    // Get the last race
    $race_nid = variable_get('ff1_last_race', 0);
  }

  // load the race
  $node = node_load(array('nid' => $race_nid));
  if (!$node) {
    drupal_set_message(t('The selected race does not exist.'));
    drupal_goto("races");
  }

  // tell the user if the results are still provisional
  if ((variable_get('ff1_result_status', t('Official')) == t('Provisional')) && ($node->nid == variable_get('ff1_last_race', 0))) drupal_set_message(t('### PROVISIONAL ###'));

  // Breadcrumb navigation
  //$breadcrumb[] = l(t('Home'), NULL);
  //drupal_set_breadcrumb($breadcrumb);

  // Set the title
  drupal_set_title(t('results : ') . $node->title);

  // Get the results from the db
  $race_results = db_query("SELECT * FROM {ff1_results_formal} WHERE race_nid = %d", $node->nid);

  if (!db_num_rows($race_results)) {
    drupal_set_message(t("No results are available for the '%race' Grand Prix.", array('%race' => $node->sentence_name)));
    drupal_goto("races");
  }

  // Race
  $result = db_query(db_rewrite_sql("SELECT f.position, f.driver_nid, d.comp_name driver, c.nid car_nid, c.comp_name car, e.nid engine_nid, e.comp_name engine, f.laps, f.time
FROM ff1_results_formal f
INNER JOIN ff1_components d ON f.driver_nid = d.nid
INNER JOIN ff1_relationships rc ON f.driver_nid = rc.nid
INNER JOIN ff1_components c ON rc.child_nid = c.nid
INNER JOIN ff1_relationships re ON rc.child_nid = re.nid
INNER JOIN ff1_components e ON re.child_nid = e.nid
WHERE f.race_nid = %d
AND f.session = 'race'
AND rc.rel_type = 'driver_car'
AND re.rel_type = 'car_engine'
ORDER BY f.position
"), $node->nid);

  // Create a table containing each of the races.
  while ($results = db_fetch_object($result)) {
    // Set up table rows
    $rows_r[] = array(
      $results->position,
      l($results->driver, 'node/' . $results->driver_nid),
      l($results->car, 'node/' . $results->car_nid),
      l($results->engine, 'node/' . $results->engine_nid),
      $results->laps,
      $results->time
    );
  };

  // Set up table header
  $header_r = array(
    t('Position'),
    t('Driver'),
    t('Car'),
    t('Engine'),
    t('Laps'),
    t('Time')
  );

  // Generate a table using: theme('table', $header, $rows)
  $output_r = theme('table', $header_r, $rows_r);

  // Fastest Lap
  $fastest_lap = db_fetch_object(db_query(db_rewrite_sql("SELECT f.driver_nid, d.comp_name driver, f.time
FROM ff1_results_formal f
INNER JOIN ff1_components d ON f.driver_nid = d.nid
WHERE f.race_nid = %d
AND f.session = 'fastest_lap'
"), $node->nid));

  $output_lap = theme('form_element', t('fastest lap'), l($fastest_lap->driver, 'node/' . $fastest_lap->driver_nid) . ' - ' . $fastest_lap->time);

  // output race and fastest lap info to a box
  $output = theme('box', 'race', $output_r . $output_lap);

  // Qualifying
  $result = db_query(db_rewrite_sql("SELECT f.position, f.driver_nid, d.comp_name driver, c.nid car_nid, c.comp_name car, e.nid engine_nid, e.comp_name engine, f.laps, f.time
FROM ff1_results_formal f
INNER JOIN ff1_components d ON f.driver_nid = d.nid
INNER JOIN ff1_relationships rc ON f.driver_nid = rc.nid
INNER JOIN ff1_components c ON rc.child_nid = c.nid
INNER JOIN ff1_relationships re ON rc.child_nid = re.nid
INNER JOIN ff1_components e ON re.child_nid = e.nid
WHERE f.race_nid = %d
AND f.session = 'qualifying'
AND rc.rel_type = 'driver_car'
AND re.rel_type = 'car_engine'
ORDER BY f.position
"), $node->nid);

  // Create a table containing each of the races.
  while ($results = db_fetch_object($result)) {
    // Set up table rows
    $rows_q[] = array(
      $results->position,
      l($results->driver, 'node/' . $results->driver_nid),
      l($results->car, 'node/' . $results->car_nid),
      l($results->engine, 'node/' . $results->engine_nid),
      $results->time
    );
  };

  // Set up table header
  $header_q = array(
    t('Position'),
    t('Driver'),
    t('Car'),
    t('Engine'),
    t('Time')
  );

  // Generate a table using: theme('table', $header, $rows)
  $output .= theme('box', 'qualifying', theme('table', $header_q, $rows_q));

  // output the page
  print theme('page', $output);
}

?>