<?php
// $Id: team.inc,v 1.4 2007/04/16 21:41:14 ff1 Exp $

/**
 * @file
 * Enables users to create a team in the fantasy formula 1 game.
 */


/**
 * Implementation of hook_node_name().
 */
function ff1_team_node_name($node) {
  return t('team');
}

/**
 * Implementation of hook_access().
 */
function ff1_team_access($op, $node) {
  global $user;

  if ($op == 'create') {
    return user_access('create teams');
  }

  if ($op == 'update' || $op == 'delete') {
    if (user_access('edit own teams') && ($user->uid == $node->uid)) {
      return TRUE;
    }
  }
}

/**
 * Implementation of hook_form().
 */
function ff1_team_form(&$node) {

  $result = db_query("SELECT * FROM {ff1_components}");
  while ($comp = db_fetch_object($result)) {
    // Turn results into an associative array for use in javascript
    $strComponentsArray = $strComponentsArray . "  componentsArray[" . $comp->nid . "] = " . $comp->price . ";\n";
  }

  //<script language="JavaScript" src="' . drupal_get_path('module', 'ff1').'/ff1.js' . ' type="text/javascript"></script>

  $cur = t('\u00A3');

  $outputscript =
'<script language="JavaScript">
  componentsArray = new Array();
  componentsArray[0]=0;
'
. $strComponentsArray .
'
function CalcTeamPrice() {
    var editdriver1 = document.getElementById(\'edit-driver1\');
    var editdriver2 = document.getElementById(\'edit-driver2\');
    var editcar = document.getElementById(\'edit-car\');
    var editengine = document.getElementById(\'edit-engine\');
    var editprice = document.getElementById(\'edit-price\');

    var driver1 = componentsArray[parseInt(editdriver1.value)];
    var driver2 = componentsArray[parseInt(editdriver2.value)];
    var car = componentsArray[parseInt(editcar.value)];
    var engine = componentsArray[parseInt(editengine.value)];
    var teamprice = driver1 + driver2 + car + engine;
    if (teamprice>' . variable_get('ff1_max_price', 100) . '){
       editprice.value = \'' . $cur . '\' + teamprice + " million -> TOO EXPENSIVE!";
    }
    else {
       var remaining = ' . variable_get('ff1_max_price', 100) . ' - teamprice;
       editprice.value = \'' . $cur . '\' + teamprice + "million -> " + \'' . $cur . '\' + remaining + "million remaining";
    }
    return teamprice;
}
</script>';

  // Add the javascript file
  drupal_set_html_head($outputscript);

  if (function_exists('taxonomy_node_form')) {
    $output .= implode('', taxonomy_node_form('ff1_team', $node));
  }

  $onchange='onchange="CalcTeamPrice();"';

  $drivers = ff1_component_arrayload('driver', TRUE);
  $team_selection = form_select(t('Driver1'), 'driver1', $node->driver1, $drivers, NULL, $onchange, FALSE, TRUE);
  $team_selection .= form_select(t('Driver2'), 'driver2', $node->driver2, $drivers, NULL, $onchange, FALSE, TRUE);

  $cars = ff1_component_arrayload('car', TRUE);
  $team_selection .= form_select(t('Car'), 'car', $node->car, $cars, NULL, $onchange, FALSE, TRUE);

  $engines = ff1_component_arrayload('engine', TRUE);
  $team_selection .= form_select(t('Engine'), 'engine', $node->engine, $engines, NULL, $onchange, FALSE, TRUE);
  $team_selection .= form_textfield(t('Team Price'), 'price', '', 20, 100, NULL, NULL, FALSE);

  $output .= form_textarea(t('Team Profile'), 'body', $node->body, 60, 5, '', NULL, FALSE);
  $output .= form_group('Team Selection', $team_selection);
  $output .= form_group('Tie Breaker',form_textfield(t('Time'), 'tiebreaker', $node->tiebreaker, 20, 11, t('Please estimate the winning time (to the nearest millisecond) of the final race of the 2007 F1 season. Example: 1:13:56.924'),NULL ,TRUE));
  $output .= filter_form('format', $node->format);
  $output .= '<script language="JavaScript">CalcTeamPrice();</script>';
  return $output;
}

function ff1_team_validate(&$node) {
  if (!isset($node->title)) {
    return;
  }

  //Check the 2 drivers are from different teams
  $car1 = db_result(db_query("SELECT child_nid FROM {ff1_relationships} WHERE nid = '%d'", $node->driver1));
  $car2 = db_result(db_query("SELECT child_nid FROM {ff1_relationships} WHERE nid = '%d'", $node->driver2));
  if ($car1 == $car2) {
    form_set_error('driver2', t('Both drivers must be from different teams.'));
  }

  //Check the car and engine are from different teams
  $engine = db_result(db_query("SELECT child_nid FROM {ff1_relationships} WHERE nid = '%d'", $node->car));
  if ($engine == $node->engine) {
    form_set_error('engine', t('Your engine must be from a different team to your car.'));
  }

  //Check the price of the team
  $teamprice = db_result(db_query("SELECT sum(price) FROM {ff1_components} WHERE nid IN ('%d', '%d', '%d', '%d')", $node->driver1, $node->driver2, $node->car, $node->engine));
  if ($teamprice > variable_get('ff1_max_price', 100)) {
    form_set_error('price', t('Your team is too expensive. Please re-select your team.'));
  }

  //Check the format of the tiebreaker
  if (empty($node->tiebreaker)) {
    form_set_error('tiebreaker', t('Please enter a value for the tiebreaker.'));
  } elseif (!_ff1_team_check_tiebreaker($node->tiebreaker)) {
    form_set_error('tiebreaker', t('Invalid tiebreaker format. Please check the format of the tiebreaker.'));
  } else {
    // ### THIS CODE IS UNTESTED ###
    // Calculate the tie breaker in seconds
    $hours = substr($str,0,1);
    $mins = substr($str,2,2);
    $secs = substr($str,5,6);
    $node->tbvalue = (3600 * $hours) + (60 * $mins) + $secs;
  }

  //Check that the team name does not already exist
  if (db_num_rows(db_query("SELECT nid FROM {node} WHERE type = '%s' AND status = 1 AND title = '%s' AND nid <> %d", $node->type, $node->title, $node->nid))) {
    form_set_error('title', t('This team name is already being used. Please choose a different team name.'));
  }
}

function _ff1_team_check_tiebreaker($str) {
  $str = StripSlashes($str);
  if (strlen($str)!=11){
     return false;
  }
  if (substr($str,1,1)!=":" || substr($str,4,1)!=":" || substr($str,7,1)!="."){
     return false;
  }
  if (!is_numeric(substr($str,0,1))) return false;
  if (!is_numeric(substr($str,2,2))) return false;
  if (!is_numeric(substr($str,5,2))) return false;
  if (!is_numeric(substr($str,8,3))) return false;

  return true;
}

/**
 * Implementation of hook_load().
 */
function ff1_team_load($node) {
  // Add the team details to the team node
  $team = db_fetch_object(db_query("SELECT * FROM {ff1_teams} WHERE nid = '%d'", $node->nid));
  // Add child components to team node
  $result = db_query("SELECT child_nid, rel_type FROM {ff1_relationships} WHERE nid = '%d'", $node->nid);
  while ($comp = db_fetch_object($result)) {
    $rel_type = $comp->rel_type;
    $child_nid = $comp->child_nid;
    $team->$rel_type = $child_nid;
  }
  // Get the total price of the team
  $team->price = db_result(db_query("SELECT sum(c.price) AS price FROM {ff1_teams} t INNER JOIN {ff1_relationships} rt ON t.nid = rt.nid INNER JOIN {ff1_components} c ON rt.child_nid = c.nid WHERE t.nid = %d GROUP BY t.nid", $node->nid));
  // Get the teams current position in the main public league
  $team->current_position = db_result(db_query("SELECT rank FROM {ff1_league_table} WHERE team_nid = %d AND race_nid = %d AND table_type = 'points_so_far'", $node->nid, variable_get('ff1_last_race', 0)));

  return $team;
}

/**
 * Implementation of hook_insert().
 */
function ff1_team_insert($node) {
  db_query("INSERT INTO {ff1_teams} (nid, team_name, tiebreaker, tbvalue) VALUES (%d, '%s', '%s', %s)", $node->nid, $node->title, $node->tiebreaker, $node->tbvalue);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver1')", $node->nid, $node->driver1);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver2')", $node->nid, $node->driver2);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'car')", $node->nid, $node->car);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'engine')", $node->nid, $node->engine);
  db_query("INSERT INTO {ff1_league_table} (race_nid, table_type, rank, team_nid, d1_score, d2_score, car_score, engine_score, team_score) VALUES (0, 'points_so_far', 0, %d, 0, 0, 0, 0, 0)", $node->nid);
}

/**
 * Implementation of hook_update().
 */
function ff1_team_update($node) {
  db_query("UPDATE {ff1_teams} SET team_name = '%s', tiebreaker = '%s', tbvalue = %s WHERE nid = %d", $node->title, $node->tiebreaker, $node->tbvalue, $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE nid = %d", $node->nid);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver1')", $node->nid, $node->driver1);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'driver2')", $node->nid, $node->driver2);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'car')", $node->nid, $node->car);
  db_query("INSERT INTO {ff1_relationships} (nid, child_nid, rel_type) VALUES (%d, '%d', 'engine')", $node->nid, $node->engine);
}

/**
 * Implementation of hook_delete().
 */
function ff1_team_delete($node) {
  db_query("DELETE FROM {ff1_teams} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_relationships} WHERE child_nid = %d", $node->nid);
  db_query("DELETE FROM {ff1_league_table} WHERE team_nid = %d", $node->nid);
}

/**
 * Implementation of hook_view().
 */
function ff1_team_view(&$node, $main = 0, $page = 0) {
  global $user;
  $node = ff1_team_content($node, $page);
  if ($page) {
    // Breadcrumb navigation
    $breadcrumb[] = l(t('Home'), NULL);
    $breadcrumb[] = l(t('teams'), 'league/table');
    drupal_set_breadcrumb($breadcrumb);
  }

  // return the team details
  return theme('node', $node, $main, $page);
}

/**
 * Implementation of hook_content().
 */
function ff1_team_content(&$node, $page = 0) {
  global $user;
  if ($page) {
    // Full page
    if ((variable_get('ff1_pre_season', 0)) && ($node->uid != $user->uid)) {
      // It is still pre--season, so cannot view other teams
      $output .= theme('box', t('Access Denied'), t('You are not allowed to view details of other teams until after the first race of the season.'));
    } else {
      // return the team details
      $output .= theme('form_element', t('Team Name'), $node->team_name);
      $output .= theme('form_element', t('Team Manager'), l($node->name,'user/' . $node->uid));
      if ($node->body) $output .= theme('form_element', t('Team Profile'), check_output($node->body, $node->format));
      $output .= theme('form_element', t('Driver1'), l(ff1_component_get_name($node->driver1),'node/' . $node->driver1));
      $output .= theme('form_element', t('Driver2'), l(ff1_component_get_name($node->driver2),'node/' . $node->driver2));
      $output .= theme('form_element', t('Car'), l(ff1_component_get_name($node->car),'node/' . $node->car));
      $output .= theme('form_element', t('Engine'), l(ff1_component_get_name($node->engine),'node/' . $node->engine));
      $output .= theme('form_element', t('Tie Breaker'), $node->tiebreaker);
      $output .= theme('form_element', t('Team Price'), t('&pound;%price million', Array('%price' => $node->price)));
      $from = (int)($node->current_position / variable_get('ff1_league_teams_per_page', 50));
      $from = $from * variable_get('ff1_league_teams_per_page', 50);
      // Display teams points
      $result = db_query(db_rewrite_sql("
SELECT r.race_name, lt.race_nid, lt2.rank, lt.team_nid, lt.d1_score, lt.d2_score, lt.car_score, lt.engine_score, lt.team_score, lt2.team_score AS total_score
FROM ff1_league_table lt
INNER JOIN ff1_teams t ON lt.team_nid = t.nid
INNER JOIN ff1_races r ON lt.race_nid = r.nid
INNER JOIN ff1_league_table lt2 ON lt.race_nid = lt2.race_nid AND lt.team_nid = lt2.team_nid
WHERE lt.team_nid = %d
AND lt.table_type = 'points_this_race'
AND lt2.table_type = 'points_so_far'
ORDER BY r.race_number ASC
"), $node->nid);
      if (db_num_rows($result)) {
        // Put results into a table
        // Set up table header
        $header = array(
          t('Race'),
          t('Position'),
          t('Driver 1'),
          t('Driver 2'),
          t('Car'),
          t('Engine'),
          t('Team Score'),
          t('Total Points')
        );
        while ($points = db_fetch_object($result)) {
          // Set up table rows
          $rows[] = array(
            l($points->race_name, 'node/' . $points->race_nid),
            $points->rank,
            $points->d1_score,
            $points->d2_score,
            $points->car_score,
            $points->engine_score,
            $points->team_score,
            $points->total_score
          );
        }
        $output .= theme('form_element', t('Points so far'), theme('table', $header, $rows));
      }
      // Get team positions in Main Public League
      $rows_pos[] = array(
        l(t('Main Public League'), 'league/table'),
        l($node->current_position, 'league/table?from=' . $from)
      );
      //$output .= theme('form_element', t('Current Position (Main Public League)'), l($node->current_position, 'league/table?from=' . $from));
      // Get team positions in other leagues
      $result = db_query(db_rewrite_sql("
SELECT l.nid AS league_nid, l.league_name, count(*) AS pos
FROM {ff1_teams} t
INNER JOIN {ff1_relationships} rt ON t.nid = rt.child_nid
INNER JOIN {ff1_relationships} rl ON rt.nid = rl.nid
INNER JOIN {ff1_league_table} lt ON rl.child_nid = lt.team_nid
INNER JOIN {ff1_leagues} l ON rl.nid = l.nid
WHERE t.nid = %d
AND lt.table_type = 'points_so_far'
AND lt.race_nid = %d
AND lt.rank <= %d
GROUP BY rl.nid
ORDER BY pos ASC
"), $node->nid, variable_get('ff1_last_race', 0), $node->current_position);
      if (db_num_rows($result)) {
        // Put results into a table
        while ($leagues = db_fetch_object($result)) {
          // Set up table rows
          $rows_pos[] = array(
            l($leagues->league_name, 'league/table/' . $leagues->league_nid),
            $leagues->pos
          );
        }
      }
      // Set up table header
      $header_pos = array(
        t('League'),
        t('Position')
      );
      $output .= theme('form_element', t('Current League Positions'), theme('table', $header_pos, $rows_pos));
    }
    $node->body = '<div class="content">'. $output . '</div></div>';
    $node->teaser = $node->body;
  }
  else {
    // Short page
    $node->teaser = check_output($node->teaser,$node->format);
  }
  $node->readmore = true;
  return $node;
}

/**
 * Custom function to display a page where you can your own teams.
 */
function ff1_team_my_teams($arg = 0) {
  $current_user = FALSE;
  if ($arg) {
    $user = db_fetch_object(db_query("SELECT uid, name FROM {users} WHERE uid = %d OR name = '%s'", $arg, $arg));
    if ($user) {
      drupal_set_title(t('Teams by %user', array('%user' => $user->name)));
    }
  }

  if (!$user) {
    global $user;
    $current_user = TRUE;
    drupal_set_title(t('My Teams'));
  }

  if (!$user->uid) {
    print theme('page', t('You need to log in to see your teams.'));
    return;
  }

  // use $user->uid to view all teams owned by user
  $result = db_query(db_rewrite_sql("
    SELECT t.*, nl.nid league_id, l.league_name FROM {ff1_teams} t
    INNER JOIN {node} nt ON nt.nid = t.nid
    LEFT JOIN {ff1_relationships} r ON r.child_nid = t.nid
    LEFT JOIN {node} nl ON nl.nid = r.nid
    LEFT JOIN {ff1_leagues} l ON l.nid = nl.nid
    WHERE nt.uid = '%d'"), $user->uid);

  if (!db_num_rows($result)) {
    print theme('page', ($current_user ? t('You have no teams.') . ' <a href=node/add/ff1_team>Create team</a>' : t('This user has no teams.')));
    return;
  }

  // Create a table containing each of the users teams, including links
  // and also including links to the leagues that team has joined.
  $team_id = 0;
  while ($node = db_fetch_object($result)) {
    // Set up table rows
    if ($team_id != $node->nid) {
      if ($team_id != 0) {
        $team_leagues = (count($leagues) > 0 ? implode('<br />', $leagues) : '');
        $rows[] = array(
          $team_id,
          $team_name,
          $team_tb,
          $team_leagues
        );
      }
      $team_id = $node->nid;
      $team_name = l($node->team_name, 'node/' . $node->nid);
      $team_tb = $node->tiebreaker;
      $leagues = array();
      if ($current_user) $leagues[] = l(t('join league'), 'node/' . $team_id . '/joinleague');
    }
    if ($node->league_id) $leagues[] = l($node->league_name, 'league/table/' . $node->league_id);
  }
  $team_leagues = (count($leagues) > 0 ? implode('<br />', $leagues) : '');
  $rows[] = array(
    $team_id,
    $team_name,
    $team_tb,
    $team_leagues
  );

  // Set up table header
  $header = array(
    t('Team ID'),
    t('Team Name'),
    t('Tie Breaker'),
    t('Leagues')
  );

  // Generate a table using: theme('table', $header, $rows)
  $output = theme('table', $header, $rows);

  print theme('page', $output);
}

/**
 * Custom function to allow users to enter their team into a league.
 */
function ff1_team_joinleague() {
  global $user;
  $node = node_load(array('nid' => arg(1)));

  if ($node->uid != $user->uid) {
      // tell the user that they do not have permission to enter this team into a league.
      drupal_set_message(t('You do not have permission to enter the team \'%team\' into a league.', array('%team' => $node->team_name)));
      // go back to the users 'my teams' page.
      drupal_goto("team/user/$node->uid");
  }

  // Breadcrumb navigation
  $breadcrumb[] = l(t('Home'),NULL);
  $breadcrumb[] = l($node->title, 'node/'. $node->nid);
  $breadcrumb[] = l(t('Join League'), 'node/'. arg(1) .'/joinleague');
  drupal_set_breadcrumb($breadcrumb);

  if (isset($_POST['submit'])) {
    $edit = $_POST['edit'];
    $league = node_load(array('nid' => $edit['league']));
    // Check the password for private leagues
    if ($league->private) {
      if (md5($edit['password']) != $league->password) {
        form_set_error('password', t('Invalid password. Please enter the correct password'));
      }
    }
    if (!form_get_errors()) {
      $team = $node->nid;
      // create the relationship.
      ff1_relationship('create', $league->nid, $team, 'league_team');
      // tell the user that his team has been added to the league.
      drupal_set_message(t('Your team \'%team\' has been added to the league \'%league\'.', array('%team' => $node->team_name, '%league' => $league->league_name)));
      // add message to watchdog log
      watchdog('special', t('team %team joined league %league.', array('%team' => '<em>'. $node->team_name .'</em>', '%league' => '<em>' . $league->league_name . '</em>')), l(t('view'), 'league/table/' . $league->nid));
      // go back to the users 'my teams' page.
      drupal_goto("team/user/$node->uid");
    }
  }
  $leagues = ff1_league_arrayload();
  $output .= theme('form_element', $node->team_name, t('Use this form to enter your team into another league.'));
  $form_output .= form_select(t('League'), 'league', '', $leagues, t('Select the league you wish to join.'), 0, FALSE, TRUE);
  $form_output .= form_password(t('Password'), 'password', '', 12, 24, t('To join a private league, you must enter the password supplied by the league owner.') . '<br />' . t('Leave this field blank to join a public league.'), NULL, FALSE);
  $form_output .= form_submit(t('submit'), 'submit');
  $output .= form($form_output);
  print theme('page', $output);
}

/*
 * THIS FUNCTION WAS ONLY USED TO UPDATE THE tbvalue FIELD WHEN THIS WAS NOT
 * DONE DURING THE TEAM FORM WITHIN ff1_team_validate.
function _ff1_team_update_tbvalue() {
  $result = db_query("SELECT * FROM {ff1_teams}");
  while ($team = db_fetch_object($result)) {
    $str = $team->tiebreaker;
    // Calculate the tie breaker in seconds
    $hours = substr($str,0,1);
    $mins = substr($str,2,2);
    $secs = substr($str,5,6);
    $team->tbvalue = (3600 * $hours) + (60 * $mins) + $secs;
    db_query("UPDATE {ff1_teams} SET tbvalue = %s WHERE nid = %d", $team->tbvalue, $team->nid);
  }
}
*/

?>